@@ -1,6 +1,6 @@
 /*
  * ansilove.c
- * Ansilove 4.1.1
+ * Ansilove 4.1.4
  * https://www.ansilove.org
  *
  * Copyright (c) 2011-2020 Stefan Vogt, Brian Cassidy, and Frederic Cambus
@@ -10,7 +10,6 @@
  * See LICENSE file for details.
  */
 
-#define _GNU_SOURCE
 #include <sys/time.h>
 #include <ansilove.h>
 #include <err.h>
@@ -24,9 +23,6 @@
 
 #ifdef HAVE_SECCOMP
 #include <sys/prctl.h>
-#include <sys/syscall.h>
-#include <linux/audit.h>
-#include <linux/filter.h>
 #include <linux/seccomp.h>
 #include "seccomp.h"
 #endif
@@ -46,7 +42,7 @@ static void
 synopsis(void)
 {
 	fprintf(stdout, "SYNOPSIS\n"
-	    "     ansilove [-dhiqrsv] [-b bits] [-c columns] [-f font]"
+	    "     ansilove [-dhiqrsSv] [-b bits] [-c columns] [-f font]"
 	    " [-m mode] [-o file]\n"
 	    "              [-R factor] [-t type] file\n");
 }
@@ -67,6 +63,7 @@ main(int argc, char *argv[])
 	/* SAUCE record related bool types */
 	bool justDisplaySAUCE = false;
 	bool fileHasSAUCE = false;
+	bool useSAUCEInfo = false;
 
 	int getoptFlag;
 
@@ -101,7 +98,7 @@ main(int argc, char *argv[])
 	if (ansilove_init(&ctx, &options) == -1)
 		errx(EXIT_FAILURE, "%s", ansilove_error(&ctx));
 
-	while ((getoptFlag = getopt(argc, argv, "b:c:df:him:o:qrR:st:v")) != -1) {
+	while ((getoptFlag = getopt(argc, argv, "b:c:df:him:o:qrR:sSt:v")) != -1) {
 		switch (getoptFlag) {
 		case 'b':
 			options.bits = strtonum(optarg, 8, 9, &errstr);
@@ -122,12 +119,6 @@ main(int argc, char *argv[])
 			break;
 		case 'f':
 			font = optarg;
-			for (size_t loop = 0; loop < FONTS; loop++) {
-				if (!strcmp(fonts[loop], font)) {
-					options.font = fontsId[loop];
-					break;
-				}
-			}
 			break;
 		case 'h':
 			synopsis();
@@ -163,6 +154,9 @@ main(int argc, char *argv[])
 		case 's':
 			justDisplaySAUCE = true;
 			break;
+		case 'S':
+			useSAUCEInfo = true;
+			break;
 		case 't':
 			type = strtolower(optarg);
 			break;
@@ -203,6 +197,122 @@ main(int argc, char *argv[])
 	}
 
 	if (!justDisplaySAUCE) {
+		/* gather rendering hints from SAUCE */
+		if (useSAUCEInfo && fileHasSAUCE) {
+			bool usedSAUCE = false;
+			if (record->dataType == 1) {
+				if (record->fileType == 0 || record->fileType == 1 || record->fileType == 2) {
+					options.columns = record->tinfo1;
+					if (record->flags & 1)
+						options.icecolors = true;
+					if ((record->flags & 6) == 4)
+						options.bits = 9;
+					if ((record->flags & 24) == 8)
+						options.dos = true;
+					usedSAUCE = true;
+				}
+				if (record->fileType == 8) {
+					options.columns = record->tinfo1;
+					usedSAUCE = true;
+				}
+			}
+			if (record->dataType == 5) {
+				options.columns = record->tinfo1;
+				usedSAUCE = true;
+			}
+			/* XBIN (dataType == 6) could also use tinfo1 for width, but we trust the XBIN header more */
+			/* font info */
+			if ((record->dataType == 1 && (record->fileType == 0 || record->fileType == 1 || record->fileType == 2)) || record->dataType == 5 ) {
+				if (strcmp(record->tinfos, "IBM VGA") == 0) {
+					font = "80x25";
+				}
+				if (strcmp(record->tinfos, "IBM VGA50") == 0) {
+					font = "80x50";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 437") == 0) {
+					font = "80x25";
+				}
+				if (strcmp(record->tinfos, "IBM VGA50 437") == 0) {
+					font = "80x50";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 775") == 0) {
+					font = "baltic";
+				}
+				if (strcmp(record->tinfos, "IBM VGA50 855") == 0) {
+					font = "cyrillic";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 863") == 0) {
+					font = "french-canadian";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 737") == 0) {
+					font = "greek";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 869") == 0) {
+					font = "greek-869";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 862") == 0) {
+					font = "hebrew";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 861") == 0) {
+					font = "icelandic";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 850") == 0) {
+					font = "latin1";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 852") == 0) {
+					font = "latin2";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 865") == 0) {
+					font = "nordic";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 860") == 0) {
+					font = "portuguese";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 866") == 0) {
+					font = "russian";
+				}
+				if (strcmp(record->tinfos, "IBM VGA 857") == 0) {
+					font = "turkish";
+				}
+				if (strcmp(record->tinfos, "Amiga MicroKnight") == 0) {
+					font = "microknight";
+				}
+				if (strcmp(record->tinfos, "Amiga MicroKnight+") == 0) {
+					font = "microknight+";
+				}
+				if (strcmp(record->tinfos, "Amiga mOsOul") == 0) {
+					font = "mosoul";
+				}
+				if (strcmp(record->tinfos, "Amiga P0T-NOoDLE") == 0) {
+					font = "pot-noodle";
+				}
+				if (strcmp(record->tinfos, "Amiga Topaz 1") == 0) {
+					font = " topaz500";
+				}
+				if (strcmp(record->tinfos, "Amiga Topaz 1+") == 0) {
+					font = "topaz500+";
+				}
+				if (strcmp(record->tinfos, "Amiga Topaz 2") == 0) {
+					font = "topaz";
+				}
+				if (strcmp(record->tinfos, "Amiga Topaz 2+") == 0) {
+					font = "topaz+";
+				}
+			}
+			if (usedSAUCE) {
+				fprintf(messages, "SAUCE info used for rendering hints\n\n");
+			}
+		}
+
+		if (font) {
+			for (size_t loop = 0; loop < FONTS; loop++) {
+				if (!strcmp(fonts[loop], font)) {
+					options.font = fontsId[loop];
+					break;
+				}
+			}
+		}
+
 		/* create output file name if output is not specified */
 		if (!output) {
 			/* appending ".png" extension to output file name */
@@ -219,6 +329,8 @@ main(int argc, char *argv[])
 		/* get file extension */
 		char *fext = strrchr(input, '.');
 		fext = fext ? strtolower(strdup(++fext)) : strdup("");
+		if (!fext)
+			errx(EXIT_FAILURE, "Memory allocation error.");
 
 		/* check if current file has a .diz extension */
 		if (!strcmp(fext, "diz"))
@@ -231,10 +343,6 @@ main(int argc, char *argv[])
 		if (fileHasSAUCE)
 			ctx.length -= 129 - (record->comments > 0 ? 5 + 64 * record->comments : 0);
 
-		/* set icecolors to true if defined in SAUCE record ANSiFlags */
-		if (fileHasSAUCE && (record->flags & 1))
-			options.icecolors = true;
-
 		int (*loader)(struct ansilove_ctx *, struct ansilove_options *) = NULL;
 
 		/* if type was specified, attempt to find a loader */
@@ -310,7 +418,7 @@ main(int argc, char *argv[])
 		fprintf(messages, "Filetype: %d\n", record->fileType);
 
 		if (record->flags)
-			fprintf(messages, "Flags: %d\n", record->flags);
+			fprintf(messages, "Flags: 0b%d%d%d%d%d%d%d%d\n", record->flags >> 7, record->flags >> 6 & 1, record->flags >> 5 & 1, record->flags >> 4 & 1, record->flags >> 3 & 1, record->flags >> 2 & 1, record->flags >> 1 & 1, record->flags & 1);
 
 		if (record->tinfo1)
 			fprintf(messages, "Tinfo1: %d\n", record->tinfo1);
